
/*
 * File MinisatInterfacing.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-25. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-25 only because so is required by the CASC-25
 * rules, and for no other reason.
 */

#ifndef __MinisatInterfacing__
#define __MinisatInterfacing__

#include "SATSolver.hpp"
#include "SATLiteral.hpp"
#include "SATClause.hpp"

#include "Minisat/core/Solver.h"

namespace SAT{

class MinisatInterfacing : public PrimitiveProofRecordingSATSolver
{
public: 
  CLASS_NAME(MinisatInterfacing);
  USE_ALLOCATOR(MinisatInterfacing);
  
	MinisatInterfacing(const Shell::Options& opts, bool generateProofs=false);

 
  virtual void addClause(SATClause* cl) override;
  
 
  virtual void simplify() override {
    CALL("MinisatInterfacing::simplify");
    _solver.simplify();
  }

  virtual Status solve(unsigned conflictCountLimit) override;
  
 
  virtual VarAssignment getAssignment(unsigned var);

 
  virtual bool isZeroImplied(unsigned var);
 
  virtual void collectZeroImplied(SATLiteralStack& acc);
 
  virtual SATClause* getZeroImpliedCertificate(unsigned var);

  virtual void ensureVarCount(unsigned newVarCnt) override;
  
  virtual unsigned newVar() override;
  
  virtual void suggestPolarity(unsigned var, unsigned pol) override {
    
    bool mpol = pol ? false : true; 
    _solver.suggestPolarity(vampireVar2Minisat(var),mpol);
  }
  
 
  virtual void addAssumption(SATLiteral lit);
  
  virtual void retractAllAssumptions() {
    _assumptions.clear();
    _status = UNKNOWN;
  };
  
  virtual bool hasAssumptions() const {
    return (_assumptions.size() > 0);
  };


  virtual void recordSource(unsigned satlitvar, Literal* lit) {
    
  };
  
  Status solveUnderAssumptions(const SATLiteralStack& assumps, unsigned conflictCountLimit, bool) override;

protected:    
  void solveModuloAssumptionsAndSetStatus(unsigned conflictCountLimit = UINT_MAX);
  
  Minisat::Var vampireVar2Minisat(unsigned vvar) {
    ASS_G(vvar,0); ASS_LE(vvar,(unsigned)_solver.nVars());
    return (vvar-1);
  }
  
  unsigned minisatVar2Vampire(Minisat::Var mvar) {
    return (unsigned)(mvar+1);
  }
  
  const Minisat::Lit vampireLit2Minisat(SATLiteral vlit) {
    return Minisat::mkLit(vampireVar2Minisat(vlit.var()),vlit.isNegative()); 
  }
  
 
  const SATLiteral minisatLit2Vampire(Minisat::Lit mlit) {
    return SATLiteral(minisatVar2Vampire(Minisat::var(mlit)),Minisat::sign(mlit) ? 0 : 1);            
  }
  
private:
  Status _status;
  Minisat::vec<Minisat::Lit> _assumptions;  
  Minisat::Solver _solver;
};

}

 #endif
